"use strict";

var initializeProxy = function (proxy, delegate, index) {
    return proxy.init(delegate, proxy.conf, index);
};

/**
 * Wraps a single middleware function in the configured proxy chain
 * @param delegate The delegate function
 * @param conf The application configuration
 * @returns {*}
 */
var proxyWrap = function (delegate, index, conf) {
    var proxies = conf.proxies;
    if (!proxies.length) {
        return delegate;
    }

    var lastProxy = initializeProxy(proxies[proxies.length - 1], delegate, index);
    for (var i = proxies.length - 2; i >= 0; i--) {
        lastProxy = initializeProxy(proxies[i], lastProxy, index);
    }
    return lastProxy;
};

module.exports = {
    process: function process(middleware, conf) {
        return middleware.map(function (fn, index) {
            return proxyWrap(fn, index, conf);
        });
    }
};